package test;

import java.io.StringReader;
import java.util.ArrayList;

import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.eclipse.persistence.internal.queries.ListContainerPolicy;
import org.eclipse.persistence.oxm.NamespaceResolver;
import org.eclipse.persistence.oxm.XMLContext;
import org.eclipse.persistence.oxm.XMLDescriptor;
import org.eclipse.persistence.oxm.mappings.XMLCompositeCollectionMapping;
import org.eclipse.persistence.oxm.mappings.XMLCompositeObjectMapping;
import org.eclipse.persistence.oxm.mappings.XMLDirectMapping;
import org.eclipse.persistence.sessions.Project;

public class Tester {
    public static void main(String[] args) {
        Tester t = new Tester();
        t.test();
    }

    public void test() {
        XMLContext xCtx = new XMLContext(new TestProject());
        org.eclipse.persistence.jaxb.JAXBContext jCtx = new org.eclipse.persistence.jaxb.JAXBContext(xCtx);

        String str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + 
        "<remc:Response xmlns:remc=\"myuri\" xmlns:common=\"mycommonuri\">" +
          "<remc:ResourceStatusLocations>" + 
            "<remc:ID>AAA</remc:ID>" + 
            "<remc:Status>0</remc:Status>" +  
            "<remc:Location>" + 
              "<common:id>AAA</common:id>" +                 
              "<common:Coordinate>" +                 
                "<common:X>5.0</common:X>" +    
                "<common:Y>5.0</common:Y>" + 
              "</common:Coordinate>" +
            "</remc:Location>" + 
            "</remc:ResourceStatusLocations>" +
            "<remc:ResourceStatusLocations>" +          
            "<remc:ID>BBB</remc:ID>" +
            "<remc:Status>2</remc:Status>" +   
            "<remc:Location>" +                 
              "<common:id>BBB</common:id>" +                 
               "<common:Coordinate>" +                  
                 "<common:X>7.0</common:X>" +   
                 "<common:Y>7.0</common:Y> " + 
              "</common:Coordinate>" +
            "</remc:Location>" +
          "</remc:ResourceStatusLocations>" +
        "</remc:Response>";

        Unmarshaller unmarshaller = jCtx.createUnmarshaller();
        Response resp = null;
        try {
            resp = (Response) unmarshaller.unmarshal(new StringReader(str));
        } catch (JAXBException e) {
            e.printStackTrace();
        }
        
        Marshaller marshaller = jCtx.createMarshaller();
        try {
            marshaller.marshal(resp, System.out);
        } catch (JAXBException e) {
            e.printStackTrace();
        }
    }
    
    class TestProject extends Project {
        public TestProject() {
            addDescriptor(getResponseDescriptor());
            addDescriptor(getResourceStatusLocationsDescriptor());
            addDescriptor(getResourceStatusDescriptor());
            addDescriptor(getResourceLocationDescriptor());
            addDescriptor(getPointDescriptor());
        }

        XMLDescriptor getResponseDescriptor() {
            // setup mappings
            XMLCompositeCollectionMapping rslmapping = new XMLCompositeCollectionMapping();
            rslmapping.setXPath("remc:ResourceStatusLocations");
            rslmapping.setReferenceClass(ResourceStatusLocations.class);
            rslmapping.setAttributeAccessor(new RSLAttributeAccessor());
            rslmapping.setContainerPolicy(new ListContainerPolicy(ArrayList.class));
            
            // setup namespace resolver
            NamespaceResolver namespaceResolver = new NamespaceResolver();
            namespaceResolver.put("remc", "myuri");
            namespaceResolver.put("common", "mycommonuri");

            // setup descriptor
            XMLDescriptor descriptor = new XMLDescriptor();
            descriptor.setJavaClass(Response.class);
            descriptor.setDefaultRootElement("remc:Response");
            descriptor.setNamespaceResolver(namespaceResolver);
            descriptor.addMapping(rslmapping);
            return descriptor;
        }

        XMLDescriptor getResourceStatusLocationsDescriptor() {
            // setup mappings
            XMLCompositeObjectMapping rsmapping = new XMLCompositeObjectMapping();
            rsmapping.setAttributeName("rs");
            rsmapping.setXPath(".");
            rsmapping.setReferenceClass(ResourceStatus.class);
            
            XMLCompositeObjectMapping rlmapping = new XMLCompositeObjectMapping();
            rlmapping.setAttributeName("rl");
            rlmapping.setXPath("remc:Location");
            rlmapping.setReferenceClass(ResourceLocation.class);
            
            // setup namespace resolver
            NamespaceResolver namespaceResolver = new NamespaceResolver();
            namespaceResolver.put("remc", "myuri");

            // setup descriptor
            XMLDescriptor descriptor = new XMLDescriptor();
            descriptor.setJavaClass(ResourceStatusLocations.class);
            descriptor.setNamespaceResolver(namespaceResolver);
            descriptor.addMapping(rsmapping);
            descriptor.addMapping(rlmapping);
            return descriptor;
        }
        
        XMLDescriptor getResourceStatusDescriptor() {
            // setup mappings
            XMLDirectMapping idmapping = new XMLDirectMapping();
            idmapping.setAttributeName("id");
            idmapping.setXPath("remc:ID/text()");
            
            XMLDirectMapping statusmapping = new XMLDirectMapping();
            statusmapping.setAttributeName("status");
            statusmapping.setXPath("remc:Status/text()");

            // setup namespace resolver
            NamespaceResolver namespaceResolver = new NamespaceResolver();
            namespaceResolver.put("remc", "myuri");

            // setup descriptor
            XMLDescriptor descriptor = new XMLDescriptor();
            descriptor.setJavaClass(ResourceStatus.class);
            descriptor.setNamespaceResolver(namespaceResolver);
            descriptor.addMapping(idmapping);
            descriptor.addMapping(statusmapping);
            return descriptor;
        }
        
        XMLDescriptor getResourceLocationDescriptor() {
            // setup mappings
            XMLDirectMapping idmapping = new XMLDirectMapping();
            idmapping.setAttributeName("id");
            idmapping.setXPath("common:id/text()");
            
            XMLCompositeObjectMapping pointmapping = new XMLCompositeObjectMapping();
            pointmapping.setAttributeName("posn");
            pointmapping.setXPath("common:Coordinate");
            pointmapping.setReferenceClass(Point.class);

            // setup namespace resolver
            NamespaceResolver namespaceResolver = new NamespaceResolver();
            namespaceResolver.put("common", "mycommonuri");

            // setup descriptor
            XMLDescriptor descriptor = new XMLDescriptor();
            descriptor.setJavaClass(ResourceLocation.class);
            descriptor.setNamespaceResolver(namespaceResolver);
            descriptor.addMapping(idmapping);
            descriptor.addMapping(pointmapping);
            return descriptor;
        }
        
        XMLDescriptor getPointDescriptor() {
            // setup mappings
            XMLDirectMapping xmapping = new XMLDirectMapping();
            xmapping.setAttributeName("X");
            xmapping.setXPath("common:X/text()");
            
            XMLDirectMapping ymapping = new XMLDirectMapping();
            ymapping.setAttributeName("Y");
            ymapping.setXPath("common:Y/text()");

            // setup namespace resolver
            NamespaceResolver namespaceResolver = new NamespaceResolver();
            namespaceResolver.put("common", "mycommonuri");

            // setup descriptor
            XMLDescriptor descriptor = new XMLDescriptor();
            descriptor.setJavaClass(Point.class);
            descriptor.setNamespaceResolver(namespaceResolver);
            descriptor.addMapping(xmapping);
            descriptor.addMapping(ymapping);
            return descriptor;
        }
    }
}
